#!/usr/bin/env python
# coding: utf-8

# # Python Lexplore Meteostation Level 0 > Level 1A

# #### Import Packages

import netCDF4
import pandas as pd
import numpy as np
import os
from datetime import datetime
from pytz import timezone

# Define functions

def adddata(data,name,date,dt,shortname,unit,longname,att,column,inpath):
    # Make sure NetCDF files are closed
    try: ncfile.close() 
    except: pass
    
    file = name+"/"+date+"_"+name.split("/")[-1]+".nc"
    
    try:
    
        # Load NetCDF file or create new
        if os.path.isfile(file):
            ncfile = netCDF4.Dataset(file,mode='a',format='NETCDF4_CLASSIC')
            # Check not duplicate run
            if float(dt) in ncfile.variables['time'][:]:
                print("Duplicate run. Exiting...")
                return
        else:
            ncfile = createnetcdf(file,shortname,unit,longname,att)

        # Add data to NetCDF file
        temp = ncfile.variables[shortname]
        temp[:] = np.ma.append(temp[:],data[column][0])
        #temp[len(temp)] = data[column][0]

        # Add timestamp to NetCDF file
        time = ncfile.variables['time']
        time[-1] = dt   

        # Close NetCDF file
        ncfile.close()

    except Exception as e: 
        # Create error log file
        if os.path.isfile(inpath+"/log.error"):
            with open(inpath+"/log.error", "a") as f:
                f.write(file+","+str(dt)+"\n")
        else:
            with open(inpath+"/log.error", "w+") as f:
                f.write(file+","+str(dt)+"\n")
        print(e)   

def createnetcdf(file,var,unit,name,att):
    
    # Create new file
    ncfile = netCDF4.Dataset(file,mode='w',format='NETCDF4_CLASSIC') 

    # Create dimensions
    depth_time = ncfile.createDimension('time', None)

    # Create variables    
    time = ncfile.createVariable('time', np.float64, ('time',))
    time.units = 'seconds since 1970-01-01 00:00:00'
    time.long_name = 'time'
    
    v = ncfile.createVariable(var,np.float64,('time')) # note: unlimited dimension is leftmost
    v.units = unit
    v.standard_name = name
        
    # Add global attributes
    for key in att:
        setattr(ncfile, key, att[key])
            
    return ncfile

def meteo(data,outfolders,inpath):
        
    # Get the timestamp and date
    dt = datetime.strptime(data["TIMESTAMP"][0], '%Y-%m-%d %H:%M:%S').replace(tzinfo=timezone('UTC')).timestamp()
    date = datetime.utcfromtimestamp(dt).strftime('%Y%m%d')
    
    # General attributes
    
    attgen = {
        "institution":"EPFL",
        "source":"Lexplore Weather station Campbell CR1000",
        "references":"LéXPLORE commun instruments sebastien.lavanchy@epfl.ch",
        "history":"See history on Renku",
        "conventions":"CF 1.7",
        "comment":"Data from the Meteostation on Lexplore Platform in Lake Geneva"
    }

    # Temperature
    folder = outfolders["airtemp"]
    att = {
        **attgen,
        "title":folder.split("/")[-1],
        "Instrument":"HC2S3 Hygroclip"
    }
    adddata(data,folder,date,dt,"airtemp","degree_C","air_temperature",att,"AirTC_Avg",inpath)

    # Wind Speed
    folder = outfolders["windspeed"]
    att = {
        **attgen,
        "title":folder.split("/")[-1],
        "Instrument":"05103-5 Wind monitor"
    }
    adddata(data,folder,date,dt,"windspeed","m s-1","wind_speed",att,"WS_ms_S_WVT",inpath)

    # Wind Direction
    folder = outfolders["winddir"]
    att = {
        **attgen,
        "title":folder.split("/")[-1],
        "Instrument":"05103-5 Wind monitor"
    }
    adddata(data,folder,date,dt,"winddir","degrees","wind_from_direction",att,"WindDir_D1_WVT",inpath)

    # Rainfall
    folder = outfolders["rain"]
    att = {
        **attgen,
        "title":folder.split("/")[-1],
        "Instrument":"ARG100 Rain gauge"
    }
    adddata(data,folder,date,dt,"rain","mm","thickness_of_rainfall_amount",att,"Rain_mm_Tot",inpath)
    
    # Temperature Pressure
    folder = outfolders["tempP"]
    att = {
        **attgen,
        "title":folder.split("/")[-1],
        "Instrument":"CS100 Setra Barometric"
    }
    adddata(data,folder,date,dt,"tempP","degree_C","pressure_temperature",att,"PTemp_C_Avg",inpath)

    # Solar Radiation
    folder = outfolders["solar"]
    att = {
        **attgen,
        "title":folder.split("/")[-1],
        "Instrument":"CS300 Pyranometer"
    } 
    adddata(data,folder,date,dt,"solar","W m-2","solar_irradiance",att,"SlrW_Avg",inpath)

# Main Script

#Testing (Local Paths)
#inpath = "data"
#outfolders = {
#    "airtemp":"out/1A0001_LexploreMeteostationTemperature",
#    "windspeed":"out/1A0002_LexploreMeteostationWindSpeed",
#    "winddir":"out/1A0003_LexploreMeteostationWindDirection",
#    "rain":"out/1A0004_LexploreMeteostationRainfall",
#    "tempP":"out/1A0005_LexploreMeteostationTemperaturePressure",
#    "solar":"out/1A0006_LexploreMeteostationSolarRadiation"
#}

#Testing (Renku Paths)
#inpath = "../../../../data/Level0/00001_ThetisDecimatedData/Incoming"
#outfolder = "../../../../data/Level1"

#Production (Server)
inpath = "data/Level0/00001_LexploreMeteostation"
outfolders = {
    "airtemp":"data/Level1A/1A0001_LexploreMeteostationTemperature",
    "windspeed":"data/Level1A/1A0002_LexploreMeteostationWindSpeed",
    "winddir":"data/Level1A/1A0003_LexploreMeteostationWindDirection",
    "rain":"data/Level1A/1A0004_LexploreMeteostationRainfall",
    "tempP":"data/Level1A/1A0005_LexploreMeteostationTemperaturePressure",
    "solar":"data/Level1A/1A0006_LexploreMeteostationSolarRadiation"
}

#Testing
#files = os.listdir("data")
#files = [inpath + "/" + s for s in files]

# Production (Server)
files = check_output("git diff-tree --no-commit-id --name-only -r "+argv[1], shell=True).decode("utf-8").splitlines()


for file in files:
    if file.rsplit("/",1)[0] == inpath and os.path.splitext(file)[1] != ".error":
        df = pd.read_csv(file, header=[1,2,3])
        for index, row in df.iterrows():
            meteo(row,outfolders,inpath)