
library(ncdf4)

CTD = function(filepath,outpath,att){
    
    # Make sure NetCDF files are closed
    tryCatch(nc_close(ncfile), error=function(e) print("No file open."))
                 
    tryCatch({
        # Read file into dataframe
        df = read.table(filepath, header = 1, sep = "\t", skip = 5)
        
        # Add depth and limit to 50m of data
        df$Depth = seq.int(nrow(df)) * 0.1
        df = head(df,500)
        
        # Get the timestamp and date
        dt = mean(df$Timestamp..s.)
        date = format(as.Date(as.POSIXct(dt, origin="1970-01-01")),"%Y%m%d");
        
        file = paste0(outpath,"/",date,"_",tail(unlist(strsplit(outpath,"[/]")),n=1),".nc")
        
        # Load NetCDF file or create new
        if (file.exists(file)){
            ncfile = nc_open(file,write=T)
            # Check not duplicate run
            if (is.element(as.numeric(dt),ncvar_get(ncfile,"time"))){
                print("Duplicate run. Exiting...")
                return("Move")
            }
        } else {
            ncfile = createnetcdf(df,file,"temp","degree_C","lake_water_temperature",att)
        }
        
        # Add time vale
        time = ncvar_get(ncfile, varid="time")
        if (time[1] == 0){
            t_vals = dt
        } else {
            t_vals = append(time, dt)
        }
        ncvar_put(ncfile, varid='time', vals=t_vals, start=c(1), count=length(t_vals))
        
        # Add data to NetCDF file
        ncvar_put( ncfile, "temp", df$Temperature, start=c(1,length(t_vals)), count=c(-1,-1))    
        
        # Close NetCDF file
        nc_close(ncfile)
        print("Succeded")

        return("Move")
    }, error=function(e){
        print(e)
        return("Stay")
    })
    
}

createnetcdf = function(df,file,var,unit,name,att){
    
    # Create dimensions
    depth_dim = ncdim_def( "depth", "m", df$Depth, unlim=FALSE )
    depth_time = ncdim_def( "time", "seconds since 1970-01-01 00:00:00", 0, unlim=TRUE )
    
    # Create variables 
    v = ncvar_def( var, unit, list(depth_dim,depth_time), longname=name)

    # Create new file
    ncfile = nc_create(file,list(v),force_v4=TRUE)
    
    # Add global attributes
    ncatt_put(ncfile,0,"title",att$title)
    ncatt_put(ncfile,0,"institution",att$institution)
    ncatt_put(ncfile,0,"source",att$source)
    ncatt_put(ncfile,0,"references",att$references)
    ncatt_put(ncfile,0,"history",att$history)
    ncatt_put(ncfile,0,"conventions",att$conventions)
    ncatt_put(ncfile,0,"comment",att$comment)
    
    return(ncfile)
}

#Testing (Local Paths)
#inpath = "data"
#prpath = "processed"
#outfolder = "out"

#Testing (Renku Paths)
#inpath = "../../../../data/Level0/00001_ThetisDecimatedData/Incoming"
#prpath = "../../../../data/Level0/00001_ThetisDecimatedData/Processed"
#outfolder = "../../../../data/Level1"

#Production (Server)
inpath = "data/Level0/00001_ThetisDecimatedData/Incoming"
prpath = "data/Level0/00001_ThetisDecimatedData/Processed"
outfolder = "data/Level1"

files = list.files(inpath)

for (file in files){
    move = "Stay"
    ft = head(unlist(strsplit(tail(unlist(strsplit(file,"[_]")),n=1),"[.]")),n=1)
    filepath = paste0(inpath,"/",file)
    if (ft == "CTD"){
        # Set attributes
        att = list()
        att[[ "title" ]] = "10001_ThetisTemperature"
        att[[ "institution" ]] = "EPFL"
        att[[ "source" ]] = "Add some source"
        att[[ "references" ]] = "Add some reference"
        att[[ "history" ]] = "See history on Renku"
        att[[ "conventions" ]] = "CF 1.7"
        att[[ "comment" ]] = "Some comment here"
        outpath = paste0(outfolder,"/10001_ThetisTemperature")
        move = CTD(filepath,outpath,att)
    }
    if (move == "Move"){
        file.rename(filepath, paste0(prpath,"/",file))
    }
}